#![cfg_attr(
    all(not(debug_assertions), target_os = "windows"),
    windows_subsystem = "windows"
)]

mod screen_color_picker;

use paris::*;
use screen_color_picker::{Screen, ScreenColorPicker};
use std::time::Instant;

// Learn more about Tauri commands at https://tauri.app/v1/guides/features/command
#[tauri::command]
fn greet(name: &str) -> String {
    format!("Hello, {}! You've been greeted from Rust!", name)
}

#[tauri::command]
fn refresh_displays() {
    match ScreenColorPicker::global().refresh_displays() {
        Ok(_) => {}
        Err(error) => {
            error!("{}", error)
        }
    }
}

#[tauri::command]
fn take_snapshot() -> Vec<String> {
    let start = Instant::now();

    let screenshots = match ScreenColorPicker::global().take_screenshots_for_all() {
        Ok(bitmaps) => {
            info!("bitmaps len: {}", bitmaps.len());
            match ScreenColorPicker::global().screens.lock() {
                Ok(screens) => Vec::from_iter(
                    screens
                        .iter()
                        .enumerate()
                        .map(|(index, screen)| bitmap_to_webp_base64(screen, &bitmaps[index])),
                ),
                Err(error) => {
                    error!("can not lock screens. {}", error);
                    Vec::<String>::new()
                }
            }
        }
        Err(error) => {
            error!("can not take screenshots for all. {}", error);
            Vec::<String>::new()
        }
    };
    println!("运行耗时: {:?}", start.elapsed());
    screenshots
}

fn bitmap_to_webp_base64(screen: &Screen, bitmap: &Vec<u8>) -> String {
    let mut bitflipped = Vec::with_capacity(screen.width * screen.height * 3);
    let stride = bitmap.len() / screen.height;

    // let mut img = Image::new(w as u32, h as u32);
    for y in 0..screen.height {
        for x in 0..screen.width {
            let i = stride * y + 4 * x;
            bitflipped.extend_from_slice(&[bitmap[i + 2], bitmap[i + 1], bitmap[i]]);
            // img.set_pixel(
            //     x as u32,
            //     y as u32,
            //     Pixel::new(buffer[i + 2], buffer[i + 1], buffer[i]),
            // );
        }
    }

    let webp_memory = webp::Encoder::from_rgb(
        bitflipped.as_slice(),
        screen.width as u32,
        screen.height as u32,
    )
    .encode(100.0);
    return base64::encode(&*webp_memory);
}

fn main() {
    tauri::Builder::default()
        .invoke_handler(tauri::generate_handler![
            greet,
            take_snapshot,
            refresh_displays
        ])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
